#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
""" Creates AMQP message templates """

import logging

# pylint: disable=E0401

from immutable_views import DictView

from workspaces.shared_interfaces import MessageArchitectIF

# pylint: disable=E0401
from workspaces.workflow.schema import WorkflowRequest

logger = logging.getLogger(__name__)

"""
Class which create an immutable dictionary of amqp message templates for easy access
"""
service_templates = DictView(
    {
        "ws": {
            "service": None,
            "routing_key": None,
            "subject": None,
        },
        "archive": {
            "service": "archive",
            "routing_key": None,
            "application": "workflow",
            "subject": None,
        },
    }
)


workflow_msg_templates = DictView(
    {
        "workflow_failed": {
            "type": "workflow-failed",
        },
        "delivery": {
            "type": "delivery",
            "delivery": None,
        },
        "ingestion_complete": {
            "type": "ingestion-complete",
        },
        "ingestion_failed": {
            "type": "ingestion-failed",
        },
        "qa_ready": {
            "type": "qa_ready",
        },
        "qa_pass": {
            "type": "qa-pass",
        },
        "qa_fail": {
            "type": "qa-fail",
        },
        "qa_complete": {
            "type": "qa-complete",
        },
        "qa_abandon": {
            "type": "qa-abandon"
        },
        "carta_ready": {
            "carta_url": None,
            "type": "carta-ready",
        },
    }
)

archive_msg_templates = DictView(
    {
        "carta_ready": {
            "carta_url": None,
            "message": "carta ready",
        },
        "carta_failed": {
            "carta_url": None,
            "message": "carta failed",
        },
        "seci_failed": {
            "message": "seci failed",
            "status": "failed",
        },
        "seci_complete": {
            "message": "seci complete",
            "status": "complete",
        },
        "qa_failed": {
            "logData": {
                "status": "FAIL",
                "type": "calibration",
                "fileset_id": None,
                "markedDirectory": None,
            }
        },
    }
)


class WorkflowMessageArchitect(MessageArchitectIF):
    """Builds workflow AMQP messages"""

    def __init__(
        self,
        request: WorkflowRequest = None,
        previous_info=None,
        delivery_info=None,
        carta_url=None,
        ui_info=None,
    ):
        self.subject = request or previous_info
        self.delivery = delivery_info
        self.carta_url = carta_url
        self.ui_info = ui_info

    @staticmethod
    def get_message_template(msg_type: str) -> dict:
        return {**service_templates["ws"], **workflow_msg_templates[msg_type]}

    def compose_message(self, msg_type: str) -> DictView:
        if any(item in msg_type for item in ["qa_, carta"]):
            service = "capability"
        else:
            service = "workflow"

        template = self.get_message_template(msg_type)
        template = template.copy()

        template["service"] = service
        template["routing_key"] = service
        template["subject"] = self.subject

        if self.delivery is not None:
            template["delivery"] = self.delivery

        if self.carta_url is not None:
            template["carta_url"] = self.carta_url

        if self.ui_info is not None:
            template["capability_version"] = self.ui_info

        return DictView(template)


class ArchiveMessageArchitect(MessageArchitectIF):
    """Builds archive system AMQP messages"""

    def __init__(self, routing_key: str, request: WorkflowRequest, carta_url: str = None, sdm_id: str = None):
        self.key = routing_key
        self.subject = request
        self.carta_url = carta_url
        self.sdm_id = sdm_id

    @staticmethod
    def get_message_template(msg_type: str) -> dict:
        return {**service_templates["archive"], **archive_msg_templates[msg_type]}

    def compose_message(self, msg_type: str) -> DictView:
        template = self.get_message_template(msg_type)
        template = template.copy()
        template["subject"] = self.subject
        template["routing_key"] = self.key

        if self.carta_url is not None:
            template["carta_url"] = self.carta_url

        if self.sdm_id is not None:
            template["fileset_id"] = self.sdm_id
            template["markedDirectory"] = self.subject.results_dir

        return DictView(template)
